# Extra ansible packages (used to get ansible-lint to find everything in
# github actions).
# See https://nens-meta.readthedocs.io/en/latest/config-files.html for info.
# (nens-meta only suggested this file, it doesn't update/manage it, btw).
---
collections:
  - ansible.posix
