root = true

[*]
end_of_line = lf
insert_final_newline = true
trim_trailing_whitespace = true
charset = utf-8
indent_style = space
indent_size = 4
max_line_length = 88

[*.{toml,yaml,yml}]
indent_size = 2

[*.{json,geojson,jsonl,js,jsx,ts,tsx,css,less,scss,html,xml}]
indent_size = 2

[*.md]
max_line_length = off

[Makefile]
indent_style = tab
