# Pyc/pyo/backup files.
*.py[co]
*~
*.swp
*.pyc
.DS_Store

# Packages/buildout/pip/pipenv
*.egg
*.egg-info
sdist
pip-log.txt
bower_components/
node_modules/
doc/build/
ansible/*.retry
.venv
venv
bin/
lib/
pyvenv.cfg
dist/
var/
*.suggestion

# Our generated files
for_step_summary.md

# Unit test / coverage reports
.coverage
.tox
htmlcov
.codeintel
coverage.*
.pytest_cache

# Pycharm, visual studio
.idea
.vscode

# Docker
docker-compose.override.yml
